/* 
 * CSE142, 99 Winter
 * Homework 5
 * Valentine's Day
 *
 * Author:     John Chandler --- chandler@u.washington.edu
 * Student ID: 9726736
 * Section:    BH-Aucoin
 *	
 *
 * Greeting Card Designer (Ultimate version) 
 *
 *  The GP142 graphics window shows some buttons and the inside
 *  or outside (cover) of a greeting card.  The idea is that you 
 *  can design what you want to card to look like by adding and moving
 *  objects: 
 *    messages, message color and position
 *    shapes,   shape color and position
 *
 *   To place a message or a greeting, you click on an appropriate button, and 
 *   then you can click on the desired position in the card.  
 */

#include "gp142.h"
#include <math.h>
#include <time.h>
	

#define TRUE    		1
#define FALSE   		0

/*
 * The state, or status you are in determines what happens when you click 
 * the card:
 * ADD SHAPE     --	you click the card, and a new object is placed there.
 *					You get into this mode by clicking an ADD button.
 *					After doing an ADD, you are put into SELECTED mode.
 * SELECTED      --	An object is selected. In this mode, the type 
 *					rotation applies to the current object.
 * ADD MESSAGE   --	click on the card and a message appears. After adding
 *					a message you are put in status Selected.
 * SELECT OBJECT -- You click on an object (or close to it's "center") and 
 *					that object is now the hotob. Then we're in SELECTED
 *					mode.
 */

#define STATUS_ADD_SHAPE 1
#define STATUS_SELECTED 2
#define STATUS_ADD_MESSAGE 3
#define STATUS_SELECT_OBJECT 4


/* Configure the appearance of the buttons on the menu bar 
 * by setting the number of pixels of button height and width
 */
#define BUTTON_HEIGHT  	35
#define BUTTON_WIDTH 	105

#define NUM_BUTTONS  6
#define SELECTED_COLOR  RED

/* Position of the card in the window */
#define PAGE_LLX (-GP142_XMAX + BUTTON_WIDTH +60)
#define PAGE_LLY (-GP142_YMAX + 100)
#define PAGE_WIDTH (GP142_XMAX - PAGE_LLX - 80)
#define PAGE_HEIGHT (GP142_YMAX - PAGE_LLY - 50)

/* For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.
 */
#define FILL 0
#define NOFILL 1


/* Types of objects we support.*/
#define OBJECT_NONE 0
#define OBJECT_SHAPE 1
#define OBJECT_MESSAGE 2
#define MAX_OBJECTS 20

/* Shape constants*/
#define MAX_SHAPES 4
#define FIRST_SHAPE_NUM 0
/* These are the kinds of shapes we support.*/
#define SHAPE_HEART 0
#define SHAPE_STAR  1
#define SHAPE_FLOWER  2
#define SHAPE_RING	3

/* Message Constants */
#define MAX_MESSAGES 5
#define FIRST_MESSAGE_NUM 0
/* Supported messages */
#define MESSAGE_LOVE 0
#define MESSAGE_LOVEYOU 1
#define MESSAGE_VAL 2
#define MESSAGE_BE 3
#define MESSAGE_TRUE 4


/*Possible places for mouse clicks*/
#define CLICK_BUTTON 1  /* You clicked on SOME button (doesn't say which one)*/
#define CLICK_CARD 2    /* You clicked in the card area */
#define CLICK_OUTSIDE 3	/* You clicked outside the card area, not on a button*/	

/* What we consider a close enough click to select an object */
#define PROXIMITY 5


/*******************  FUNCTION PROTOTYPES **************************/

/* Drawing functions */
void DrawBlankScreen (void);
void DrawButtons(int last_button);
void DrawCard (int background_color, int status, int object_count, 
			   int object_type[], int object_num[], 
			   int object_color[], int object_x[],
			   int object_y[]);

/* Functions involved with drawing a shape */
void DrawShape(int color, int shape_type, int bottom_x, int bottom_y,
			   int background_color);
void DrawHeart (int x, int y, int color);
void DrawStar (int x, int y, int color);
void DrawFlower (int x, int y, int color);
void DrawRing (int x, int y, int color, int background_color);

/* Functions involved with adding a message */
void DrawMessage(int message_type, int color, int bottom_x, int bottom_y);

/* Mouse click handling functions */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int object_type[], 
					  int object_num[], int object_color[], 
					  int object_x[], int object_y[], int *hotob,
					  int *last_button);

int ClassifyMouseClick(int mouse_x, int mouse_y);

void HandleButton(int x, int y, int *background_color, int *status,
				  int *object_count, int object_type[], 
				  int object_num[], int object_color[],
				  int *hotob, int *last_button);

/* Add a shape to the card. */
void AddShape(int x, int y, int *status, int *object_count, 
			   int object_type[], int object_num[], int object_color[], 
			   int object_x[], int object_y[], int *hotob);

/* Add a message to the card. */
void AddMessage(int x, int y, int *status, int *object_count, 
			   int object_type[], int object_num[], int object_color[], 
			   int object_x[], int object_y[], int *hotob);

/* This function takes the mouse x and y coordinates and changes
 * hotob to the first object that is within PROXIMITY of the click
 */
void SelObject(int x, int y,int object_count,int object_type[], 
			   int object_x[], int object_y[], 
			   int *hotob, int *status);

int distance(int x1, int y1, int x2, int y2);
/*****************************************************************************

     MAIN

*****************************************************************************/

/* main() has two chief purposes: to contain the important variables and 
 *initialize them, and to contain the "event loop," which constantly looks 
 *to see if an event has happened, and then processes the event by calling 
 *an appropriate function. An example of an event is a mouse click.
 */
int main(void)
{
	int quit = FALSE;              /* becomes TRUE to exit program */
	int mouse_x, mouse_y;          /* coordinates of a mouse click */
	char key_hit;                  /* which key was hit            */
	
	
	/*variables and their default values*/
	int background_color = YELLOW;
	int status = STATUS_ADD_SHAPE;
	
	/* The variable and arrays below are used to hold the information
	 * about the various objects (shapes and messages).
	 */

	int object_count = 0;           /* How many objects on the page? */
	
	
	int object_type[MAX_OBJECTS] ;  /* Can be a shape or message	 */
	int object_num[MAX_OBJECTS];    /* If type is shape, this is b/t 1
									 * and 4. If type is message this 
									 * is between 1 and 5.			 */
	int object_color[MAX_OBJECTS];  /* Color of object               */
	int object_x[MAX_OBJECTS];      /* Position of object            */
	int object_y[MAX_OBJECTS];
	
	int hotob = 0;					/* This will mark the object that is
									 * currently at the mercy of the mouse.
									 * (Stands for "hot object".)
									 */


	int last_button = -1;           /* Keep track of the last button clicked.
	                                   This would be usefull when highlighting
	                                   the last button the user clicked */

	int event; /*the most recent GP142 event*/
	int i; /* a counter */

	time_t t; /*needed only when initializing a random number generator*/
	

	/*
	 * INITIALIZATIONS
	 */
	 for (i=0;i<MAX_OBJECTS;i++)   /* Initializing the object_type array */
	 	object_type[i] = OBJECT_NONE;
	 
	 
	 srand (time(&t));	    /* Seed the random number generator. This is 
							   something you should do if you plan to use 
							   random numbers. Leave it here in your program 
							   and forget about it.*/
	

	/* These GP142 initializations don't need to be changed */
	GP142_open();           /* Open and init the GP142 Graphics Window      */
	GP142_show_text(0);     /* Hide the text window                         */
	GP142_logging(LOG_OFF); /* Change to LOG_ON if you want it.             */
	
	GP142_clear();	        /* This will clear the window & paint it white. */
	GP142_animate(ANI_RUN); /* Enable periodic events                       */
	
	
	
	/*
	 * The main event loop:
	 * --- ---- ----- -----
	 * wait for the next user action, such as a mouse click or keyboard
	 * keypress, or a periodic update event (allowing us to display the
	 * next frame of an animation -- not used directly in this program)
	 * and call the appropriate function to handle it.  Repeat this until 
	 * the user signals "quit".
	 */
	while (!quit) {  /* Beginning of the event loop */
		
		/*start by redrawing everything */
		DrawBlankScreen ();
		DrawCard (background_color, status, object_count, 
			      object_type, object_num, object_color, object_x, 
				  object_y);
		DrawButtons(last_button);	  /* Show the buttons...*/ 
		
		/*Wait for something to happen */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);
		
		/* Something just happened.  Figure out what it was, and react 
		   appropriately. */
		switch (event) {
		case GP142_QUIT:
			quit = TRUE;           /* set flag to terminate loop           */
			break;
			
		case GP142_KBD:  /*A key was pressed */
			break;
			
		case GP142_MOUSE:
			/* A mouse click has occurred. Call the mouse handler to 
			   decide where the click was and react as needed. */
			HandleMouseClick(mouse_x, mouse_y, &background_color, &status, 
					         &object_count, object_type, 
							 object_num, object_color, 
							 object_x, object_y,&hotob,
							 &last_button);			
			break;
			
		case GP142_PERIODIC:	
			break;

		default:
			
			break;
			
		}  /*end switch */
		
	}  /* end event loop */
	
	

	GP142_close();         /* clean up and close graphics window   */
	
	return 0;
}  /*end main */


/*****************************************************************************

     FUNCTIONS

*****************************************************************************/
/* 
 * For a blank screen just draw a white rectangle that covers the whole
 * GP142 window.
 */
void DrawBlankScreen (void) {
	int font_size = 14;

	GP142_rectangleXY (WHITE,
					-GP142_XMAX, -GP142_YMAX,
					GP142_XMAX, GP142_YMAX,
					FILL);

} /*end DrawBlankScreen */


/* 
 * Draws the buttons 
 */
void DrawButtons(int last_button)
{

	char b_labels [NUM_BUTTONS][25] = {
		"Background color",
			"Object Color",
			"Next type please",
			"Add Message",
			"Add Shape",
			"Select Object"
	};
	
	int i, text_offset = -GP142_XMAX + 6, font_size = 12;
	int y, label_y;
	
	/* draw rectangles and labels for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		y = GP142_YMAX - i * BUTTON_HEIGHT;
		
		GP142_rectangleXY(BLACK,
			-GP142_XMAX + 4,
			y - 2,
			-GP142_XMAX + BUTTON_WIDTH,
			y - BUTTON_HEIGHT - 2, 1);
		
		
		label_y = y - BUTTON_HEIGHT/2;
		
		if (i == last_button)
			GP142_printfXY(SELECTED_COLOR, text_offset, label_y, font_size, b_labels[i]);	
		else
			GP142_printfXY(BLACK, text_offset, label_y, font_size, b_labels[i]);	
	}	
}


/*
 * Render (draw) the page and all its contents.
 */
void DrawCard (int background_color, int status, int object_count, 
			   int object_type[], int object_num[], 
			   int object_color[], int object_x[], 
			   int object_y[])
{
	int font_size = 14;
	int i; /* A counter */
	/*
	 * Draw the background of the card.
	 */
	
	GP142_rectangleXY (background_color,
		PAGE_LLX, PAGE_LLY,
		PAGE_LLX+ PAGE_WIDTH, PAGE_LLY+ PAGE_HEIGHT,
		FILL);
	
	/* 
	 * Draw the objects on the card, if one currently exists.
	 */
	for (i = 0; i < object_count; i++)
	{
		if (object_type[i] == OBJECT_SHAPE)
		{
			DrawShape (object_num[i], object_color[i], object_x[i], 
				   object_y[i],background_color);
		}
		else if (object_type[i] == OBJECT_MESSAGE)
		{
			DrawMessage (object_num[i], object_color[i], object_x[i], 
				   object_y[i]);
		}
	}

	/*
	 * Write an appropriate status message below the background
	 */
	switch (status) {
	case STATUS_ADD_SHAPE:
		GP142_printfXY (GREEN, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a shape");
		break;
	case STATUS_SELECTED:
		GP142_printfXY (RED, PAGE_LLX, PAGE_LLY-40, font_size-3,
			"Object selected. Click button to change type or color.");
		break;
	case STATUS_ADD_MESSAGE:
		GP142_printfXY(GREEN, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on card to add message.");
		break;
	case STATUS_SELECT_OBJECT:
		GP142_printfXY(PEACH, PAGE_LLX, PAGE_LLY-40, font_size-1,
			"Click near the origin of the object to re-select.");
		break;
	default:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Status error -- notify programmer");
		break;
	} /*end switch */
	
	
} /*end DrawPage */


/*
 * Draw one shape.
 */
void DrawShape(int shape_type, int color, int bottom_x, int bottom_y,int background_color) {

	switch ((shape_type - FIRST_SHAPE_NUM) % MAX_SHAPES) {
	case SHAPE_HEART:
		DrawHeart (bottom_x, bottom_y, color);
		break;
	case SHAPE_STAR:
		DrawStar (bottom_x, bottom_y, color);
		break;
	case SHAPE_FLOWER:
		DrawFlower (bottom_x, bottom_y, color);
		break;
	case SHAPE_RING:
		DrawRing (bottom_x, bottom_y, color,background_color);
		break;
	default:
		/* Unknown shape selected */
		break;

	} /*end switch*/

}


/*
 * Draw a heart on the card. A heart will be made by two overlapping
 * circles and a triangle.
 *
 * The pulsation is caused by adding a small, random amount to the 
 * heart height each time this function is called.
 */
#define HEART_WIDTH 35
void DrawHeart (int x, int y, int color) {

	int h = (HEART_WIDTH/2) + (rand () %4);

	GP142_circleXY (color, x - h + 5, y, h);
	GP142_circleXY (color, x + h - 5, y, h);
	GP142_triangleXY (color, x- h*2 +5, y-5,
						x + h*2 -5, y-5,
						x, y - h*2 -5, FILL);

}


/*
 * Draw a star on the card.
 *
 * We make the star twinkle by selecting the number of points randomly.  Each 
 * time it is redrawn, it is slightly different than the time before, which 
 * makes it appear to pulsate,
 */
#define STARPOINTS 5
#define STARRADIUS 40
#define PI 3.1416
void DrawStar (int x, int y, int color) {

	int sp;
	double angle;
	int x1, y1;

	sp = STARPOINTS + (rand () %3);
	for (angle = 0.0; angle < 2 * PI; angle = angle + (2*PI)/sp) {
		x1 = (int) (cos (angle) * STARRADIUS);
		y1 = (int) (sin (angle) * STARRADIUS);
		GP142_lineXY (color, x+x1, y+y1, x-x1, y-y1, 1);

	}
}


/* Draw a flower on the card
 *
 * Draws a rose looking flower on the card. No random pulsing, yet.
 */
#define OFFSET 10 /* This is x offset that determines the central axis
				  * of the flower and the height of the ring in DrawRing */
#define STEM_HEIGHT 20
void DrawFlower (int x, int y, int color) 
{
	int radius = 8;
	int stem_color; /* color of the stem */
	stem_color = rand() % 100;

	if (stem_color % 13 == 0) /* I like Forrest green the best         */
		stem_color = GREEN;  /* so I made it happen most of the time. */
	else if (stem_color % 23 == 0)
			stem_color = SEA_GREEN;
		else if (stem_color % 17 == 0)
				stem_color = OLIVE;
			 else 
				 stem_color = FOREST_GREEN;
	
	if (stem_color != FOREST_GREEN) /* When the stem changes color, 
									 * so does the flower, occasionally */
		color = color + (rand() % 2); 
	
	GP142_lineXY (stem_color, x+OFFSET, y, x+OFFSET, y+STEM_HEIGHT, 3);
	GP142_circleXY (color, x+OFFSET, y+STEM_HEIGHT+radius ,radius);
	GP142_triangleXY (color, x+OFFSET, y + STEM_HEIGHT +2, x - 2,
					  y+STEM_HEIGHT+2*radius, x + OFFSET*2 +2,
					  y+STEM_HEIGHT+2*radius,FILL);
	 
}

/* This function draws a ring. Use sparingly! */
void DrawRing (int x, int y, int color,int background_color)
{
	GP142_circleXY(color, x, y, OFFSET+4);
	GP142_circleXY(background_color, x, y, OFFSET+1);
	GP142_triangleXY(color+4,x, y+OFFSET+3,
					x - 4, y + OFFSET + 7,
					x + 4, y + OFFSET + 7,FILL);
	GP142_lineXY (rand() % 20, x - 6, y + OFFSET +9,
							   x -8, y + OFFSET + 11,1);
	GP142_lineXY (rand() % 20, x , y + OFFSET +9,
							   x , y + OFFSET + 12,2);
	GP142_lineXY (rand() % 20, x + 6, y + OFFSET +8,
							   x + 8, y + OFFSET + 11,1);
}

/*
 * Mouse clicks occur in one of three regions of interest: on buttons, 
 * on the card, or anywhere else.
 */
int  ClassifyMouseClick(int mouse_x, int mouse_y)
{
  if ((mouse_x < -GP142_XMAX + BUTTON_WIDTH) &&
      (mouse_y < GP142_YMAX) &&
      (mouse_y > (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT)))
    return CLICK_BUTTON;
  else if (mouse_x >= PAGE_LLX && mouse_x < PAGE_LLX + PAGE_WIDTH
	  && mouse_y >= PAGE_LLY && mouse_y < PAGE_LLY + PAGE_HEIGHT)
    return CLICK_CARD;
  else
    return CLICK_OUTSIDE;

}
/* This function adds a message to the card */
void
DrawMessage(int message_type, int color, int x, int y)
{
	int font_size = 14;
	int font_rand; /* A random number that we can use
				    * to make the message pulse. Kind of 
					* annoying, so I only made two of the 
					* messages work this way.
					*/
	font_rand = (rand() % 3) - 1;
	/* Kind of makes the message look like it's twitching! */

	switch((message_type - FIRST_MESSAGE_NUM) % MAX_MESSAGES){
	case MESSAGE_LOVE:
		GP142_printfXY(color, x, y, font_size + font_rand,
			"Love 4 Evah!"); /* A true boston message of love */
		break;
	case MESSAGE_LOVEYOU:
		GP142_printfXY(color, x, y, font_size,
			"I love you"); 
		break;
	case MESSAGE_VAL:
		GP142_printfXY(color, x, y, font_size,
			"Happy Valentine's Day"); 
		break;
	case MESSAGE_BE:
		GP142_printfXY(color, x, y, font_size + font_rand,
			"Be Mine"); 
		break;
	case MESSAGE_TRUE:
		GP142_printfXY(color, x, y, font_size,
			"TRUE LOVE"); 
		break;
	default:
		break;
	}
}




/*
 * Decide where a mouse click occured, and react appropriately.
 */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int object_type[], 
					  int object_num[], int object_color[], 
					  int object_x[], int object_y[], int *hotob,
					  int *last_button)
{

	int mouse_click;
	mouse_click = ClassifyMouseClick(mouse_x, mouse_y);
	
	switch (mouse_click){
	case CLICK_BUTTON:
		/* A button has been clicked.  Call a function to handle it*/
		
		HandleButton(mouse_x, mouse_y, background_color, status, 
					 object_count, object_type, object_num,
					 object_color, hotob, last_button);
		break;
	case CLICK_CARD: /* Nested Switch statements, a true rite-of-passage */
		switch (*status){
		case STATUS_ADD_SHAPE:
		case STATUS_SELECTED:
			AddShape(mouse_x, mouse_y, status, object_count, 
					object_type, object_num, object_color,
					object_x,object_y,hotob);
			break;
		case STATUS_ADD_MESSAGE:
			AddMessage(mouse_x, mouse_y, status, object_count, 
					   object_type, object_num, object_color,
					   object_x, object_y, hotob);
			break;
		case STATUS_SELECT_OBJECT:
			SelObject(mouse_x, mouse_y, *object_count, object_type,
					  object_x, object_y, hotob, status);
			break;
		default:
			break;
		}
		
		break; /* This break goes with case: CLICK_CARD */
	default:
		break;
	}

	/* Ignore any other mouse clicks */
}

/* 
 * Figure out which button was clicked, if any, and do the right 
 * thing.
 */
void HandleButton(int x, int y, int *background_color, int *status,
				  int *object_count, int object_type[], 
				  int object_num[], int object_color[],
				  int *hotob, int *last_button)
{
	int buttonNum = 0;
	int i =1;
	/* If we're outside of this area, we could not have clicked on
	* a button.
	*/
	if (x > -GP142_XMAX + BUTTON_WIDTH)  
		return ;	/*That's an error situation*/
	
				   /* We know the mouse clicked in the column of buttons. Calculate
					* which one.
					*/
	buttonNum = (GP142_YMAX - y)/BUTTON_HEIGHT;	 
	*last_button = buttonNum;

	switch(buttonNum) {
	case 0:
		/* Change the background color */
		*background_color = (*background_color + 1) % MAX_COLORS;
		break;
	case 1:
		/* This button should change the color of the object */
		object_color[*hotob] = 
			(object_color[*hotob] + 1) % MAX_COLORS;
		break;
	case 2:
		/* Change the type of the currently selected object */
		if (object_type[*hotob] == OBJECT_SHAPE) 
			object_num[*hotob] = 
				(object_num[*hotob] + 1) % MAX_SHAPES;
		if (object_type[*hotob] == OBJECT_MESSAGE)
			object_num[*hotob] = 
				(object_num[*hotob] + 1) % MAX_MESSAGES;

		break;
	case 3:
		/* This button permits the adding of a message */
		*status = STATUS_ADD_MESSAGE;
		break;
	case 4:
		/* Button 5 allows the addition of another shape */
		*status = STATUS_ADD_SHAPE;
		break;
	
	case 5:
		/* This button should re-select the object */
		*status = STATUS_SELECT_OBJECT;
		break;
	default:
		*status = 999;
		/*Bad news if this is ever reached! */
		break;
	}
}


/* If there's room for another shape, add it to the card. */
void AddShape(int x, int y, int *status, int *object_count, 
			   int object_type[], int object_num[], int object_color[], 
			   int object_x[], int object_y[], int *hotob)
{
	if (*object_count <= MAX_OBJECTS) 
	{
		if (*status == STATUS_ADD_SHAPE && *object_count < MAX_OBJECTS)
		{
			*status = STATUS_SELECTED;
		
			*hotob = *object_count;
			object_type[*hotob] = OBJECT_SHAPE;
			object_num[*hotob] = SHAPE_HEART;
			object_color[*hotob] = PURPLE;
			object_x[*hotob] = x;
			object_y[*hotob] = y;
			*object_count += 1;
		}else if (*status == STATUS_SELECTED)
			{ /* We come here if we're just moving an object around */
				object_x[*hotob] = x;
				object_y[*hotob] = y;
			}

	}
}

/* If there's room for another message, add it to the card. */
void 
AddMessage(int x, int y, int *status, int *object_count, 
		   int object_type[], int object_num[], int object_color[], 
		   int object_x[], int object_y[],int *hotob)
{
	if (*object_count <= MAX_OBJECTS)
	{
		if (*status == STATUS_ADD_MESSAGE && *object_count < MAX_OBJECTS)
		{
			*status = STATUS_SELECTED;

			*hotob = *object_count;
			object_type[*hotob] = OBJECT_MESSAGE;
			object_num[*hotob] = MESSAGE_LOVE;
			object_color[*hotob] = BLACK;
			object_x[*hotob] = x;
			object_y[*hotob] = y;
			*object_count += 1; /* Increment the object count */
		}
		else if (*status == STATUS_SELECTED)
				{
					object_x[*hotob] = x;
					object_y[*hotob] = y;
				}
	}
}

/* This function is called by HandleMouseClick. It takes the x,y from the
 * mouse and scans through the arrays to find the first object (that's not
 * of type OBJECT_NONE) that is withing PROXIMITY of mouse_x, mouse_y
 */
void
SelObject(int x, int y, int object_count, int object_type[], 
		  int object_x[], int object_y[], int *hotob, int *status)
{
	int i; /* A counter */
	int best_so_far = 32000; /* This is the best distance so far */
	int compare;
	for (i = 0; i <= object_count; i++)
	{
		if (object_type[i] != OBJECT_NONE)
		{
			compare = distance(x,y,object_x[i],object_y[i]);
			if (compare < best_so_far && compare < (PROXIMITY * PROXIMITY))
			{	*hotob = i;
				*status = STATUS_SELECTED;
			}
		}
	}
}
	
/* This really returns the distance squared */
int distance (int x1, int y1, int x2, int y2)
{
	int x_diff, y_diff;
	x_diff = x1-x2;
	y_diff = y1-y2;
	return (x_diff*x_diff + y_diff*y_diff);
}
